| Safe Haskell | None |
|---|---|
| Language | GHC2021 |
Skeletest.Plugin
Synopsis
- data Plugin = Plugin {
- cliFlags :: [Flag]
- snapshotRenderers :: [SnapshotRenderer]
- hooks :: Hooks
- defaultPlugin :: Plugin
- data Hooks = Hooks {
- hookRunTest :: TestInfo -> IO TestResult -> IO TestResult
- defaultHooks :: Hooks
- data TestResult = TestResult {}
- data TestResultMessage
- data TestInfo = TestInfo {
- testModule :: Text
- testContexts :: [Text]
- testName :: Text
- testMarkers :: [SomeMarker]
- testFile :: FilePath
- findMarker :: IsMarker a => [SomeMarker] -> Maybe a
- hasMarkerNamed :: String -> [SomeMarker] -> Bool
Documentation
Constructors
| Plugin | |
Fields
| |
Hooks
Constructors
| Hooks | |
Fields
| |
defaultHooks :: Hooks Source #
Re-exports
TestResult
data TestResult Source #
Constructors
| TestResult | |
Fields | |
data TestResultMessage Source #
TestInfo
Constructors
| TestInfo | |
Fields
| |
Markers
findMarker :: IsMarker a => [SomeMarker] -> Maybe a Source #
Find the first marker in the given list with the given type.
hasMarkerNamed :: String -> [SomeMarker] -> Bool Source #
Return true if the given marker name is present.